@@ -57,7 +57,7 @@ def check_backend_available(backend_name):
         else:
             k, m = (10, 4)
         try:
-            driver = ECDriver(ec_type=backend_name, k=k, m=m)
+            ECDriver(ec_type=backend_name, k=k, m=m)
         except ECDriverError:
             return False
         return True
@@ -66,6 +66,7 @@ def check_backend_available(backend_name):
 def PyECLibVersion(z, y, x):
     return (((z) << 16) + ((y) << 8) + (x))
 
+
 PYECLIB_MAJOR = 1
 PYECLIB_MINOR = 1
 PYECLIB_REV = 2
@@ -142,8 +143,22 @@ class PyECLib_FRAGHDRCHKSUM_Types(PyECLibEnum):
 
 # Main ECDriver class
 class ECDriver(object):
+    '''A driver to encode, decode, and reconstruct erasure-coded data.'''
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, **kwargs):
+        '''
+        :param ec_type: the erasure coding type to use for this driver.
+        :param k: number of data fragments to use. Required.
+        :param m: number of parity fragments to use. Required.
+        :param chksum_type:
+        :param validate: default: False
+        :param library_import_str: default: 'pyeclib.core.ECPyECLibDriver'
+
+        You must provide either ``ec_type`` or ``library_import_str``;
+        typically you just want to use ``ec_type``. See ALL_EC_TYPES for the
+        list of all EC types supported by PyECLib, and VALID_EC_TYPES for the
+        list of all EC types currently available on this system.
+        '''
         self.k = -1
         self.m = -1
         self.hd = -1
@@ -156,6 +171,11 @@ class ECDriver(object):
                 raise ECDriverError(
                     "Invalid Argument: %s is required" % required)
 
+        if 'ec_type' not in kwargs and 'library_import_str' not in kwargs:
+            raise ECDriverError(
+                "Invalid Argument: either ec_type or library_import_str "
+                "must be provided")
+
         for (key, value) in kwargs.items():
             if key == "k":
                 try:
@@ -556,4 +576,5 @@ def _liberasurecode_version():
             'this message' % version_str)
     return version_str
 
+
 LIBERASURECODE_VERSION = _liberasurecode_version()
