@@ -30,6 +30,7 @@ import unittest
 
 from distutils.version import StrictVersion
 from itertools import combinations
+import six
 
 import pyeclib.ec_iface
 from pyeclib.ec_iface import ECBackendNotSupported
@@ -119,22 +120,29 @@ class TestPyECLibDriver(unittest.TestCase):
     def tearDown(self):
         pass
 
-    def test_invalid_km_args(self):
+    def test_missing_required_args(self):
+        # missing ec_type
+        with self.assertRaises(ECDriverError) as err_context:
+            ECDriver(k=1, m=1)
+        self.assertEqual(str(err_context.exception),
+                         "Invalid Argument: either ec_type or "
+                         "library_import_str must be provided")
+
         for ec_type in VALID_EC_TYPES:
             # missing k
             with self.assertRaises(ECDriverError) as err_context:
                 ECDriver(ec_type=ec_type, m=1)
-
             self.assertEqual(str(err_context.exception),
                              "Invalid Argument: k is required")
 
             # missing m
             with self.assertRaises(ECDriverError) as err_context:
                 ECDriver(ec_type=ec_type, k=1)
-
             self.assertEqual(str(err_context.exception),
                              "Invalid Argument: m is required")
 
+    def test_invalid_km_args(self):
+        for ec_type in VALID_EC_TYPES:
             with self.assertRaises(ECDriverError) as err_context:
                 # k is smaller than 1
                 ECDriver(ec_type=ec_type, k=-100, m=1)
@@ -152,10 +160,10 @@ class TestPyECLibDriver(unittest.TestCase):
         available_ec_types = []
         for _type in ALL_EC_TYPES:
             try:
-                if _type is 'shss':
+                if _type == 'shss':
                     _k = 10
                     _m = 4
-                elif _type is 'libphazr':
+                elif _type == 'libphazr':
                     _k = 4
                     _m = 4
                 else:
@@ -176,12 +184,12 @@ class TestPyECLibDriver(unittest.TestCase):
                 print("Skipping test for %s backend" % _type)
                 continue
             try:
-                if _type is 'shss':
-                    _instance = ECDriver(k=10, m=4, ec_type=_type)
-                elif _type is 'libphazr':
-                    _instance = ECDriver(k=4, m=4, ec_type=_type)
+                if _type == 'shss':
+                    ECDriver(k=10, m=4, ec_type=_type)
+                elif _type == 'libphazr':
+                    ECDriver(k=4, m=4, ec_type=_type)
                 else:
-                    _instance = ECDriver(k=10, m=5, ec_type=_type)
+                    ECDriver(k=10, m=5, ec_type=_type)
             except ECDriverError:
                 self.fail("%s algorithm not supported" % _type)
 
@@ -784,28 +792,31 @@ class TestPyECLibDriver(unittest.TestCase):
                          (baseline_usage, new_usage))
 
 
-class TestBackendsEnabled(unittest.TestCase):
+class BackendsEnabledMetaclass(type):
+    def __new__(meta, cls_name, cls_bases, cls_dict):
+        for ec_type in ALL_EC_TYPES:
+            def dummy(self, ec_type=ec_type):
+                if ec_type not in VALID_EC_TYPES:
+                    raise unittest.SkipTest
+                if ec_type == 'shss':
+                    k = 10
+                    m = 4
+                elif ec_type == 'libphazr':
+                    k = 4
+                    m = 4
+                else:
+                    k = 10
+                    m = 5
+                ECDriver(k=k, m=m, ec_type=ec_type)
+            dummy.__name__ = 'test_%s_available' % ec_type
+            cls_dict[dummy.__name__] = dummy
+        return type.__new__(meta, cls_name, cls_bases, cls_dict)
+
+
+class TestBackendsEnabled(six.with_metaclass(BackendsEnabledMetaclass,
+                                             unittest.TestCase)):
     '''Based on TestPyECLibDriver.test_valid_algo above, but these tests
        should *always* either pass or skip.'''
-    class __metaclass__(type):
-        def __new__(meta, cls_name, cls_bases, cls_dict):
-            for ec_type in ALL_EC_TYPES:
-                def dummy(self, ec_type=ec_type):
-                    if ec_type not in VALID_EC_TYPES:
-                        raise unittest.SkipTest
-                    if ec_type == 'shss':
-                        k = 10
-                        m = 4
-                    elif ec_type == 'libphazr':
-                        k = 4
-                        m = 4
-                    else:
-                        k = 10
-                        m = 5
-                    ECDriver(k=k, m=m, ec_type=ec_type)
-                dummy.__name__ = 'test_%s_available' % ec_type
-                cls_dict[dummy.__name__] = dummy
-            return type.__new__(meta, cls_name, cls_bases, cls_dict)
 
 
 if __name__ == '__main__':
